@@ -111,7 +111,7 @@ static void
 dia_clock_init (DiaClock *item)
 {
 	item->sec = 0;
-	item->timer = g_timeout_add (200, clock_update, item);
+	item->timer = 0; // g_timeout_add (200, clock_update, item);
 
 	item->circle = dia_shape_new (DIA_SHAPE_ELLIPSE);
 	item->hours = dia_shape_new (DIA_SHAPE_PATH);
