@@ -717,7 +717,7 @@ main (int argc, char* argv[])
 	DiaCanvasItem	*box = 0;
 	DiaPoint p1 = {0.0, 0.0}, p2 = {60.0, 40.0}, p3 = {40.0, 60.0};
 	DiaRectangle extents = { -300.0, -300.0, 300.0, 300.0 };
-	GdkPixbuf *pixbuf;
+	//GdkPixbuf *pixbuf;
 	GError *error = NULL;
 	//PangoContext *context;
 	PangoFontDescription *font_desc;
@@ -736,10 +736,10 @@ main (int argc, char* argv[])
 	
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 
-	pixbuf = gdk_pixbuf_new_from_file ("./image.png", &error);
-	if (error) {
-		g_error ("Could not load image.png");
-	}
+	//pixbuf = gdk_pixbuf_new_from_file ("./image.png", &error);
+	//if (error) {
+	//	g_error ("Could not load image.png");
+	//}
 
 	canvas = g_object_new (DIA_TYPE_CANVAS,
 	//			"snap_to_grid", TRUE,
@@ -866,6 +866,7 @@ main (int argc, char* argv[])
 	dia_canvas_item_connect (box, item->handles->data);
 
 	/* Add an image */
+#if WITH_IMAGE_PNG
 	g_message ("Adding an image of %f x %f.", 
 			     (gdouble) gdk_pixbuf_get_width (pixbuf),
 			     (gdouble) gdk_pixbuf_get_height (pixbuf));
@@ -876,7 +877,7 @@ main (int argc, char* argv[])
 			     NULL);
 	dia_canvas_groupable_add (DIA_CANVAS_GROUPABLE (canvas->root), item);
 	g_object_weak_ref (G_OBJECT (item), item_destroyed, NULL);
-
+#endif
 	//dia_canvas_item_rotate (item, 154.0);
 	dia_canvas_item_move (item, 100.0, 100.0);
 	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
@@ -938,6 +939,13 @@ main (int argc, char* argv[])
 		      "editable", FALSE,
 		      "markup", TRUE,
 		      NULL);
+        if (1) {
+		double affine[6];
+		art_affine_identity (affine);
+		art_affine_rotate (affine, 30);
+		dia_shape_text_set_affine (DIA_CANVAS_TEXT (DIA_TEXTBOX (item)->text)->text_shape, affine);
+	}
+
 	dia_canvas_item_move (item, 10.0, 10.0);
 	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
 	g_object_unref (item);
