@@ -275,7 +275,7 @@ dia_canvas_element_align_handles (DiaCanvasElement *element)
 {
 	DiaCanvasItem *item = DIA_CANVAS_ITEM (element);
 
-	//g_message (__FUNCTION__": w %f h %f", el->width, el->height);
+	//g_message (G_STRLOC": w %f h %f", el->width, el->height);
 
 	dia_handle_set_pos_i (GET_HANDLE (item, DIA_HANDLE_NW), 0.0, 0.0);
 
@@ -476,7 +476,7 @@ dia_canvas_element_handle_motion_rotate (DiaCanvasItem *item, DiaHandle *handle,
 
 	if (pinned) {
 		alpha = (((int) (alpha / M_PI * 180.0)) / 5) * 5;
-		//g_message (__FUNCTION__": alpha = %f", alpha);
+		//g_message (G_STRLOC": alpha = %f", alpha);
 		alpha = alpha * M_PI / 180.0;
 	}
 
@@ -709,6 +709,8 @@ dia_canvas_element_connect (DiaCanvasItem *item, DiaHandle *handle)
 	g_assert (hbegin != NULL);
 	g_assert (hend != NULL);
 
+	dia_handle_remove_all_constraints (handle);
+
 	dia_handle_add_line_constraint (hbegin, hend, handle);
 
 	return TRUE;
