@@ -418,34 +418,39 @@ dia_canvas_group_lower_item (DiaCanvasGroup *group,
 
 /**
  * dia_canvas_group_foreach:
- * @item: 
+ * @group: 
  * @func: 
  * @data: 
  *
- * Call the function @func for every child of @item. You can pass extra data
+ * DEPRICATED
+ *
+ * Call the function @func for every child of @group. You can pass extra data
  * via the @data value.
  *
  * Return value: 
  **/
 gint
-dia_canvas_group_foreach (DiaCanvasItem *item,
+dia_canvas_group_foreach (DiaCanvasGroup *group,
                           DiaCanvasItemForeachFunc func, gpointer data)
 {
-        GList *l;
+        DiaCanvasItem *item;
         gint result = FALSE;
+	DiaCanvasIter iter;
 
-        g_return_val_if_fail (DIA_IS_CANVAS_ITEM (item), FALSE);
+        g_return_val_if_fail (DIA_IS_CANVAS_ITEM (group), FALSE);
         g_return_val_if_fail (func != NULL, FALSE);
 
-        if (func (item, data))
+        if (func ((DiaCanvasItem*) group, data))
                 return TRUE;
 
-	if (!DIA_IS_CANVAS_GROUP (item))
+	if (!DIA_IS_CANVAS_GROUPABLE (group))
 		return FALSE;
 
-        for (l = DIA_CANVAS_GROUP (item)->children; l != NULL; l = l->next)
-		result |= dia_canvas_group_foreach ((DiaCanvasItem*) l->data,
+	if (dia_canvas_groupable_get_iter ((DiaCanvasGroupable*) group, &iter)) do {
+		item = dia_canvas_groupable_value ((DiaCanvasGroupable*) group, &iter);
+		result |= dia_canvas_group_foreach ((DiaCanvasGroup*) item,
 						    func, data);
+	} while (dia_canvas_groupable_next ((DiaCanvasGroupable*) group, &iter));
 
 	return result;
 }
