@@ -49,8 +49,10 @@ dia_canvas_iter_destroy(DiaCanvasIter *iter)
 {
 	g_return_if_fail (iter != NULL);
 
-	if (iter->destroy_func)
+	if (iter->destroy_func) {
 		iter->destroy_func(iter);
+		iter->destroy_func = NULL;
+	}
 	dia_canvas_iter_init(iter);
 }
 
