@@ -525,6 +525,9 @@ dia_canvas_line_set_property (GObject *object, guint property_id,
 				DiaPoint hpos, pos1, pos2;
 				DiaHandle *hbegin, *hend;
 
+				/* Make the handle move undoable. */
+				dia_handle_preserve_state (handle);
+
 				dia_handle_remove_all_constraints (handle);
 				dia_handle_get_pos_w (handle,
 						      &hpos.x, &hpos.y);
@@ -544,13 +547,12 @@ dia_canvas_line_set_property (GObject *object, guint property_id,
 					hbegin = hnew;
 					hend = h2;
 				}
-				/* Make the handle move undoable. */
-				dia_canvas_preserve_property_last (item->canvas,
-							      G_OBJECT (handle),
-							      "pos_i");
-				dia_canvas_preserve_property_last (item->canvas,
-							      G_OBJECT (handle),
-							      "connect");
+				//dia_canvas_preserve_property_last (item->canvas,
+				//			      G_OBJECT (handle),
+				//			      "pos_i");
+				//dia_canvas_preserve_property_last (item->canvas,
+				//			      G_OBJECT (handle),
+				//			      "connect");
 				dia_handle_add_line_constraint (hbegin, hend, handle);
 				/* Store the newly connected handle */
 				handles_conn = get_connected_handles (hend);
@@ -625,6 +627,9 @@ dia_canvas_line_set_property (GObject *object, guint property_id,
 				dia_canvas_item_affine_point_w2i (item,
 								  &p2.x, &p2.y);
 
+				/* Make the handle move undoable. */
+				dia_handle_preserve_state (handle);
+
 				dia_handle_remove_all_constraints (handle);
 				dia_distance_line_point (&p1, &p3, &p2,
 							 line->line_width,
@@ -632,13 +637,12 @@ dia_canvas_line_set_property (GObject *object, guint property_id,
 				dia_canvas_item_affine_point_i2w (item,
 								  &pnew.x,
 								  &pnew.y);
-				/* Make the handle move undoable. */
-				dia_canvas_preserve_property_last (item->canvas,
-							      G_OBJECT (handle),
-							      "pos_i");
-				dia_canvas_preserve_property_last (item->canvas,
-							      G_OBJECT (handle),
-							      "connect");
+				//dia_canvas_preserve_property_last (item->canvas,
+				//			      G_OBJECT (handle),
+				//			      "pos_i");
+				//dia_canvas_preserve_property_last (item->canvas,
+				//			      G_OBJECT (handle),
+				//			      "connect");
 				dia_handle_set_pos_w (handle, pnew.x, pnew.y);
 				dia_handle_add_line_constraint (h1, h3, handle);
 			}
@@ -1062,7 +1066,7 @@ calc_closest_point (DiaCanvasItem *item, gdouble x, gdouble y,
 	DiaCanvasLine *line = (DiaCanvasLine*) item;
 	gdouble d, dist = G_MAXDOUBLE;
 	gint s;
-	DiaPoint c;
+	DiaPoint c = { 0, 0 };
 	DiaPoint p1, p2;
 	DiaPoint cursor;
 	GList *l;
@@ -1160,6 +1164,8 @@ dia_canvas_line_connect (DiaCanvasItem *item, DiaHandle *handle)
 	if (!parent_class->connect (item, handle))
 		return FALSE;
 
+	//g_message (G_STRLOC":connect");
+
 	/* If the handle was previously connected to the same object, find
 	 * its 'q_connected_handles' item and remove it from it. */
 	if (handle->connected_to == item) {
