@@ -19,12 +19,14 @@
 
 #include <gtk/gtk.h>
 #include "dia-canvas-view.h"
+#include "dia-canvas-iter.h"
 #include "dia-canvas-groupable.h"
 #include "dia-canvas-editable.h"
 #include "dia-handle-layer.h"
 #include "dia-shape.h"
 #include "dia-shape-art.h"
-#include "dia-shape-x.h"
+//#include "dia-shape-x.h"
+#include "dia-shape-view-info.h"
 #include <libart_lgpl/art_uta_svp.h>
 #include <libart_lgpl/art_svp_vpath.h>
 #include <libart_lgpl/art_svp_ops.h>
@@ -35,6 +37,14 @@
 //#define D(msg) G_STMT_START { g_print (G_STRLOC": "); g_print msg; g_print ("\n"); } G_STMT_END
 #define D(msg)
 
+/**
+ * uta_dup:
+ * @uta: 
+ *
+ * Make a copy of @uta.
+ *
+ * Return value: A new ArtUta instance.
+ **/
 static ArtUta*
 uta_dup (ArtUta *uta)
 {
@@ -237,6 +247,7 @@ dia_canvas_view_item_init (DiaCanvasViewItem *item)
 	item->redraw_area = NULL;
 	item->n_handle_pos = 0;
 	item->handle_pos = NULL;
+	item->view_info = NULL;
 	item->event_time = GDK_CURRENT_TIME;
 }
 
@@ -290,21 +301,21 @@ dia_canvas_view_item_set_property (GObject *object, guint property_id,
 			/* Connect some extra signals if DiaCanvasItem is
 			 * editable...
 			 */
-			if (DIA_IS_CANVAS_EDITABLE (citem)) {
-				g_signal_connect_after (G_OBJECT (citem),
-					"start_editing",
-					G_CALLBACK (edit_item_start_editing_cb),
-					item);
-				g_signal_connect_after (G_OBJECT (citem),
-					"editing_done",
-					G_CALLBACK (edit_item_editing_done_cb),
-					item);
-			}
+			//if (DIA_IS_CANVAS_EDITABLE (citem)) {
+				//g_signal_connect_after (G_OBJECT (citem),
+				//	"start_editing",
+				//	G_CALLBACK (edit_item_start_editing_cb),
+				//	item);
+				//g_signal_connect_after (G_OBJECT (citem),
+				//	"editing_done",
+				//	G_CALLBACK (edit_item_editing_done_cb),
+				//	item);
+			//}
 			/* If our item is a group, notify the CanvasViewItem
 			 * that a new object has to be added, if a new object
 			 * is added to the DiaCanvas. */
 			if (DIA_IS_CANVAS_GROUPABLE (citem)) {
-				//g_message (__FUNCTION__": Adding group signals");
+				//g_message (G_STRLOC": Adding group signals");
 				g_signal_connect_after (G_OBJECT (citem),
 					"add",
 					G_CALLBACK (group_item_add_cb),
@@ -366,7 +377,7 @@ dia_canvas_view_item_dispose (GObject *object)
 {
 	DiaCanvasViewItem *vitem = (DiaCanvasViewItem*) object;
 
-	//g_message ("%s: begin (%x)", __FUNCTION__, (int)object);
+	//g_message ("%s: begin (%x)", G_STRLOC, (int)object);
 
 	if (vitem->item) {
 		empty_canvas_view_item (vitem);
@@ -566,6 +577,7 @@ item_has_state_cb (DiaCanvasItem *item, gint state, DiaCanvasViewItem *vitem)
 	return retval ? TRUE : FALSE;
 }
 
+#if 0
 /*
  * Signals for DiaCanvasEditables
  */
@@ -589,6 +601,7 @@ edit_item_editing_done_cb (DiaCanvasEditable *editable,
 			   const gchar *text, DiaCanvasViewItem *vitem)
 {
 }
+#endif
 
 /*
  * Signals for DiaCanvasGroupables
@@ -603,7 +616,7 @@ group_item_add_cb (DiaCanvasItem *group, DiaCanvasItem *item,
 	g_assert (DIA_IS_CANVAS_ITEM (item));
 	g_assert (GNOME_IS_CANVAS_ITEM (vitem));
 
-	//g_message (__FUNCTION__);
+	//g_message (G_STRLOC);
 	if (item->parent == group)
 		dia_canvas_view_item_add_items (GNOME_CANVAS_GROUP (vitem), item);
 }
@@ -623,7 +636,7 @@ group_item_remove_cb (DiaCanvasItem *group, DiaCanvasItem *item,
 	if (item->parent == group)
 		return;
 
-	//g_message (__FUNCTION__);
+	//g_message (G_STRLOC);
 	for (l = vgroup->item_list; l != NULL; l = l->next) {
 		if (DIA_CANVAS_VIEW_ITEM (l->data)->item == item) {
 			vitem = l->data;
@@ -637,6 +650,7 @@ group_item_remove_cb (DiaCanvasItem *group, DiaCanvasItem *item,
 	gtk_object_destroy (GTK_OBJECT (vitem));
 }
 
+/*
 static void
 remove_view_info_from_shapes (DiaCanvasViewItem *item)
 {
@@ -651,10 +665,13 @@ remove_view_info_from_shapes (DiaCanvasViewItem *item)
 			dia_shape_view_info_remove (shape, view_info);
 	} while (dia_canvas_item_shape_next (item->item, &iter));
 }
+*/
 
 static void
 empty_canvas_view_item (DiaCanvasViewItem *item)
 {
+	GSList *l;
+
 	if (!item->item)
 		return;
 
@@ -673,7 +690,13 @@ empty_canvas_view_item (DiaCanvasViewItem *item)
 	/* Remove view info from shapes!
 	 * Do this for normal state, for selected, focused and grabbed states.
 	 */
-	remove_view_info_from_shapes (item);
+	//remove_view_info_from_shapes (item);
+	for (l = item->view_info; l != NULL; l = g_slist_next(l)) { 
+		//g_message (G_STRLOC": freeing view_info %p from item %p", l->data, item);
+		dia_shape_view_info_free (item, l->data);
+	}
+	g_slist_free (item->view_info);
+	item->view_info = NULL;
 
 	/* Request a redraw for the removed area: */
 	if (item->redraw_area) {
@@ -693,25 +716,48 @@ empty_canvas_view_item (DiaCanvasViewItem *item)
 	item->item = NULL;
 }
 
-static guint32
-event_get_time (GdkEvent *event)
+update_view_info (DiaCanvasViewItem *item)
 {
-	if (event)
-		switch (event->type) {
-			case GDK_MOTION_NOTIFY:
-			       return event->motion.time;
-			case GDK_BUTTON_PRESS:
-			case GDK_2BUTTON_PRESS:
-			case GDK_3BUTTON_PRESS:
-			case GDK_BUTTON_RELEASE:
-			       return event->button.time;
-			case GDK_KEY_PRESS:
-			case GDK_KEY_RELEASE:
-			       return event->key.time;
-			default:
-			       break;
+	GSList *old_view_info = item->view_info;
+	GSList *new_view_info = NULL;
+	GSList *l, *next;
+	gboolean found;
+	DiaShapeViewInfo *view_info;
+	DiaCanvasIter iter;
+	DiaCanvasItem *diaitem = item->item;
+
+	if (dia_canvas_item_get_shape_iter (diaitem, &iter)) do {
+		DiaShape *shape = dia_canvas_item_shape_value (diaitem, &iter);
+
+		if (shape) {
+			// for each item in the old list
+			// check if the shape is the current shape
+			// if so, unlink the element,
+			// add it to the new list
+			// if no entry was found, create a new one
+			found = FALSE;
+			for (l = old_view_info; l != NULL; l = next) {
+				next = l->next;
+				if (((DiaShapeViewInfo*) l->data)->shape == shape) {
+					found = TRUE;
+					old_view_info = g_slist_remove_link (old_view_info, l);
+					new_view_info = g_slist_concat (new_view_info, l);
+					break;
+				}
+			}
+			if (!found)
+				new_view_info = g_slist_append (new_view_info, dia_shape_view_info_new(shape));
 		}
-	return GDK_CURRENT_TIME;
+	} while (dia_canvas_item_shape_next (diaitem, &iter));
+
+	// Remove all remaining entries in old_view_info
+	for (l = old_view_info; l != NULL; l = l->next) {
+		dia_shape_view_info_free (item, l->data);
+	}
+	if (old_view_info)
+		g_slist_free (old_view_info);
+
+	item->view_info = new_view_info;
 }
 
 /**
@@ -750,8 +796,6 @@ dia_canvas_view_item_update (GnomeCanvasItem *item,
 	DiaCanvasIter iter;
 	gboolean free_clip = FALSE;
 	ArtSVP *new_clip;
-	/* Prototype for the update function. */
-	ArtSVP* (* update_func) (DiaShape*, DiaCanvasViewItem*, double*, ArtSVP*, int);
 
 	D((" "));
 
@@ -766,19 +810,12 @@ dia_canvas_view_item_update (GnomeCanvasItem *item,
 
 	g_assert (DIA_IS_CANVAS_ITEM (diaitem));
 
-	//g_message (__FUNCTION__": BoundingBox (%x): %f, %f, %f, %f", 
+	//g_message (G_STRLOC": BoundingBox (%x): %f, %f, %f, %f", 
 	//(int)diaitem, item->x1, item->y1, item->x2, item->y2);
 
 	dia_handle_layer_update_handles ((DiaHandleLayer*) DIA_CANVAS_VIEW (canvas)->handle_layer,
 					 cvitem);
 
-	/* Update the shapes drawn. */
-	if (item->canvas->aa) {
-		update_func = dia_shape_art_update;
-	} else {
-		update_func = dia_shape_x_update;
-	}
-
 	//dia_canvas_view_item_update_item_state (cvitem);
 
 	if (cvitem->redraw_area) {
@@ -787,12 +824,15 @@ dia_canvas_view_item_update (GnomeCanvasItem *item,
 		cvitem->redraw_area = NULL;
 	}
 
+	/* Update our view_info */
+	update_view_info (cvitem);
+
 	if (dia_canvas_item_get_shape_iter (diaitem, &iter)) do {
 		DiaShape *shape = dia_canvas_item_shape_value (diaitem, &iter);
 
 		if (shape) {
 			/* update render info */
-			new_clip = update_func (shape,
+			new_clip = dia_shape_art_update (shape,
 						DIA_CANVAS_VIEW_ITEM (item),
 						affine, clip, flags);
 
@@ -882,6 +922,7 @@ static void
 dia_canvas_view_item_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			   int x, int y, int width, int height)
 {
+#ifdef DIACANVAS_ENABLE_X
 	DiaCanvasIter iter;
 	DiaCanvasItem *diaitem;
 
@@ -911,6 +952,7 @@ dia_canvas_view_item_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 
 	if (GNOME_CANVAS_ITEM_CLASS (parent_class)->draw)
 		GNOME_CANVAS_ITEM_CLASS (parent_class)->draw (item, drawable, x, y, width, height);
+#endif
 }
 
 static void
@@ -1058,7 +1100,7 @@ dia_canvas_view_item_bounds (GnomeCanvasItem *item,
  * Return value: TRUE if the event is handled, FALSE otherwise.
  **/
 gboolean
-dia_canvas_view_item_emit_event (DiaCanvasViewItem *item, DiaEvent *event)
+dia_canvas_view_item_emit_event (DiaCanvasViewItem *item, gpointer event)
 {
 	static guint event_signal_id = 0;
 	gboolean retval = FALSE;
@@ -1067,13 +1109,13 @@ dia_canvas_view_item_emit_event (DiaCanvasViewItem *item, DiaEvent *event)
 	g_return_val_if_fail (DIA_IS_CANVAS_ITEM (item->item), FALSE);
 	g_return_val_if_fail (event != NULL, FALSE);
 
-	if (event_signal_id == 0)
-		event_signal_id = g_signal_lookup ("event",
-						   DIA_TYPE_CANVAS_ITEM);
+	//if (event_signal_id == 0)
+	//	event_signal_id = g_signal_lookup ("event",
+	//					   DIA_TYPE_CANVAS_ITEM);
 
 	//dia_canvas_view_item_update_item_state (item);
 
-	g_signal_emit (G_OBJECT (item->item), event_signal_id, 0, event, &retval);
+	//g_signal_emit (G_OBJECT (item->item), event_signal_id, 0, event, &retval);
 
 	//dia_canvas_view_item_reset_item_state (item);
 
@@ -1287,4 +1329,3 @@ dia_canvas_view_item_is_selected (DiaCanvasViewItem *item)
 	return g_list_find (sel_items, item) ? TRUE : FALSE;
 
 }
-
