@@ -28,8 +28,8 @@
 #include "dia-default-tool.h"
 #include "dia-handle-layer.h"
 #include "dia-canvas-i18n.h"
-#include "dia-shape-art.h"
-#include "dia-shape-x.h"
+//#include "dia-shape-art.h"
+//#include "dia-shape-x.h"
 #include "diamarshal.h"
 
 enum {
@@ -355,7 +355,7 @@ dia_canvas_view_destroy (GtkObject *object)
 {
 	DiaCanvasView *view = (DiaCanvasView*) object;
 	
-	//g_message (__FUNCTION__": begin (%x).", (guint) object);
+	//g_message (G_STRLOC": begin (%x).", (guint) object);
 	
 	if (view->tool) {
 		g_object_unref (G_OBJECT (view->tool));
@@ -381,7 +381,7 @@ dia_canvas_view_destroy (GtkObject *object)
 		dia_canvas_view_unset_canvas (view);
 	}
 
-	//g_message (__FUNCTION__": calling parent.");
+	//g_message (G_STRLOC": calling parent.");
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 
 }
@@ -426,7 +426,7 @@ dia_canvas_view_button_press (GtkWidget *widget, GdkEventButton *event)
 	if (view->tool && dia_tool_button_press (view->tool, view, event))
 		return TRUE;
 
-	dia_canvas_push_undo (view->canvas, NULL);
+	//dia_canvas_push_undo (view->canvas, NULL);
 
 	if (view->default_tool && dia_tool_button_press (view->default_tool, view, event))
 		return TRUE;
@@ -831,7 +831,7 @@ dia_canvas_view_unset_canvas (DiaCanvasView *view)
 	g_return_if_fail (DIA_IS_CANVAS_VIEW (view));
 	g_return_if_fail (DIA_IS_CANVAS (view->canvas));
 
-	//g_message (__FUNCTION__);
+	//g_message (G_STRLOC);
 
 	//if (view->root_item)
 	//	g_object_remove_weak_pointer (G_OBJECT (view->root_item),
@@ -851,7 +851,7 @@ dia_canvas_view_unset_canvas (DiaCanvasView *view)
  * dia_canvas_view_new:
  * @canvas: A #DiaCanvas that contains data that is to be visualized (may
  * 		be NULL).
- * @aa: Use Anti-alias (LibArt) mode rather that X.
+ * @aa: OBSOLETE. An Anti-alias canvas is created anyway.
  *
  * Create a new DiaCanvasView object. In most cases you want to set @aa to TRUE,
  * since it has the most complete renderer and the best quality.
@@ -864,16 +864,11 @@ dia_canvas_view_new (DiaCanvas *canvas, gboolean aa)
 	GtkWidget *view;
   
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
-	if (aa) {
-		view = g_object_new (DIA_TYPE_CANVAS_VIEW,
-				     "aa", TRUE,
-				     "canvas", canvas,
-				     NULL);
-	} else {
-		view = g_object_new (DIA_TYPE_CANVAS_VIEW,
-				     "canvas", canvas,
-				     NULL);
-	}
+	view = g_object_new (DIA_TYPE_CANVAS_VIEW,
+			     "aa", TRUE,
+			     "canvas", canvas,
+			     NULL);
+
 	gtk_widget_pop_colormap ();
 	
 	g_assert (view != NULL);
@@ -1179,7 +1174,7 @@ void
 dia_canvas_view_focus (DiaCanvasView *view, DiaCanvasViewItem *item)
 {
 	gboolean retval = TRUE;
-	DiaEvent ev;
+	//DiaEvent ev;
 
 	g_return_if_fail (DIA_IS_CANVAS_VIEW (view));
 	g_return_if_fail ((item == NULL) || DIA_IS_CANVAS_VIEW_ITEM (item));
@@ -1189,8 +1184,8 @@ dia_canvas_view_focus (DiaCanvasView *view, DiaCanvasViewItem *item)
 
 	/* Unfocus focused object */
 	if (view->focus_item) {
-		ev.type = DIA_EVENT_FOCUS_OUT;
 		// TODO: What to do with focus in/out events?
+		//ev.type = DIA_EVENT_FOCUS_OUT;
 		//dia_canvas_view_item_emit_event (view->focus_item, &ev);
 		//gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (view->focus_item));
 		dia_canvas_item_request_update (view->focus_item->item);
@@ -1203,7 +1198,7 @@ dia_canvas_view_focus (DiaCanvasView *view, DiaCanvasViewItem *item)
 	if (item && DIA_CANVAS_ITEM_INTERACTIVE (item->item)) {
 		view->focus_item = item;
 		dia_canvas_view_select (view, item);
-		ev.type = DIA_EVENT_FOCUS_IN;
+		//ev.type = DIA_EVENT_FOCUS_IN;
 		//dia_canvas_view_item_emit_event (item, &ev);
 	} else {
 		view->focus_item = NULL;
@@ -1252,6 +1247,7 @@ dia_canvas_view_move (DiaCanvasView *view, gdouble dx, gdouble dy,
 
 		if (!DIA_CANVAS_ITEM_GET_CLASS (item)->move
 		    || (l->data == (gpointer) originator)
+		    || !item->parent
 		    || parent_in_selected_items_list (view->selected_items,
 						      l->data))
 			continue;
@@ -1351,8 +1347,9 @@ void
 dia_canvas_view_gdk_event_to_dia_event (DiaCanvasView *view,
 					DiaCanvasViewItem *item,
 					GdkEvent *gdk_event,
-					DiaEvent *dia_event)
+					gpointer dia_event)
 {
+	/*
 	switch (gdk_event->type) {
 	case GDK_BUTTON_PRESS:
 		dia_event->type = DIA_EVENT_BUTTON_PRESS;
@@ -1424,6 +1421,7 @@ dia_canvas_view_gdk_event_to_dia_event (DiaCanvasView *view,
 	default:
 		break;
 	}
+	*/
 }
 
 /**
@@ -1528,7 +1526,8 @@ text_buffer_changed_cb (GtkTextBuffer *buffer, DiaCanvasView *view)
  * dia_canvas_view_start_editing:
  * @view:
  * @item:
- * @text_shape:
+ * @x:
+ * @y:
  *
  * Start interactive editing of the text in @text_shape. This function
  * starts the editing widget (a #GtkTextView) and puts it over the text
@@ -1541,18 +1540,34 @@ text_buffer_changed_cb (GtkTextBuffer *buffer, DiaCanvasView *view)
  **/
 void
 dia_canvas_view_start_editing (DiaCanvasView *view, DiaCanvasViewItem *item,
-			       DiaShapeText *text_shape)
+			       gdouble x, gdouble y)
 {
 	GtkWidget *text_view;
 	GtkTextBuffer *text_buffer;
+	GtkTextIter start, end;
+	GtkTextMark *mark;
+	DiaShapeText *text_shape = NULL;
 
 	g_return_if_fail (DIA_IS_CANVAS_VIEW (view));
 	g_return_if_fail (DIA_IS_CANVAS_VIEW_ITEM (item));
-	g_return_if_fail (text_shape != NULL);
-	g_return_if_fail (((DiaShape*) text_shape)->type == DIA_SHAPE_TEXT);
+	g_return_if_fail (DIA_IS_CANVAS_EDITABLE (item->item));
+	//g_return_if_fail (text_shape != NULL);
+	//g_return_if_fail (((DiaShape*) text_shape)->type == DIA_SHAPE_TEXT);
 	g_return_if_fail (view->edited_item == NULL);
 	g_return_if_fail (view->edited_shape == NULL);
 
+	/* Let's see if we may edit and if there is a text shape to use as
+	 * a blueprint
+	 */
+	if (dia_canvas_editable_is_editable (DIA_CANVAS_EDITABLE (item->item))) {
+		gdouble ix = x, iy = y;
+		dia_canvas_item_affine_point_w2i (item->item, &ix, &iy);
+		text_shape = dia_canvas_editable_get_editable_shape (DIA_CANVAS_EDITABLE (item->item),
+								     ix, iy);
+		if (!text_shape)
+			return;
+	}
+
 	/* If the item has the `grab' on events, make sure to ungrab it first
 	 */
 	if (GNOME_CANVAS (view)->grabbed_item
@@ -1586,6 +1601,12 @@ dia_canvas_view_start_editing (DiaCanvasView *view, DiaCanvasViewItem *item,
 	if (text_shape->font_desc)
 		gtk_widget_modify_font (text_view, text_shape->font_desc);
 
+	/* select the text */
+	gtk_text_buffer_get_bounds (text_buffer, &start, &end);
+	gtk_text_buffer_place_cursor (text_buffer, &start);
+	mark = gtk_text_buffer_get_selection_bound (text_buffer);
+	gtk_text_buffer_move_mark (text_buffer, mark, &end);
+
 	/* We still have a reference to our text buffer */
 	g_object_unref (text_buffer);
 
