@@ -682,6 +682,9 @@ real_find_objects_in_rectangle (DiaCanvasItem *item, ArtDRect *rect)
  * Note that the root object is not included in the selection, only
  * non-composite leaf objects are selected.
  *
+ * NOTE: the items returned do not have their reference count incremented.
+ * So do not g_object_unref() them.
+ *
  * Return value: A list of objects that are within the rectangle.
  **/
 GList*
@@ -768,17 +771,19 @@ typedef struct {
 	DiaCanvasItem *item;
 } GlueHandleHelper;
 
-static gint
+static void
 real_glue_handle (DiaCanvasItem *item, GlueHandleHelper *helper)
 {
 	gdouble x, y, dist;
+	DiaCanvasIter iter;
+	DiaCanvasItem *child;
 
 	if (!DIA_CANVAS_ITEM_VISIBLE (helper->handle->owner))
-		return TRUE;
+		return;
 
 	/* Do not check yourself. */
 	if (helper->handle->owner == item)
-		return FALSE;
+		return;
 
 	if (DIA_CANVAS_ITEM_GET_CLASS (item)->glue) {
 		x = helper->orig_x;
@@ -792,7 +797,14 @@ real_glue_handle (DiaCanvasItem *item, GlueHandleHelper *helper)
 			helper->item = item;
 		}
 	}
-	return FALSE;
+
+	if (DIA_IS_CANVAS_GROUPABLE (item)
+	    && dia_canvas_groupable_get_iter (DIA_CANVAS_GROUPABLE (item), &iter)) do {
+		child = dia_canvas_groupable_value (DIA_CANVAS_GROUPABLE (item), &iter);
+		real_glue_handle(child, helper);
+	} while (dia_canvas_groupable_next (DIA_CANVAS_GROUPABLE (item), &iter));
+
+	return;
 }
 
 /**
@@ -833,7 +845,9 @@ dia_canvas_glue_handle (DiaCanvas *canvas, const DiaHandle *handle,
 	helper.dist = G_MAXDOUBLE;
 	helper.item = NULL;
 
-	dia_canvas_group_foreach (canvas->root, (DiaCanvasItemForeachFunc) real_glue_handle, &helper);
+	// TODO: remove depricated function.
+	//dia_canvas_group_foreach (canvas->root, (DiaCanvasItemForeachFunc) real_glue_handle, &helper);
+	real_glue_handle(canvas->root, &helper);
 
 	*glue_x = helper.glue_x;
 	*glue_y = helper.glue_y;
