@@ -270,11 +270,20 @@ print_image (DiaShapeImage *shape, GnomePrintContext *context)
 			   (gdouble) gdk_pixbuf_get_width (shape->pixbuf),
 			   (gdouble) -gdk_pixbuf_get_height (shape->pixbuf));
 	gnome_print_setopacity (context, 1.0);
-	gnome_print_rgbaimage (context,
-			       gdk_pixbuf_get_pixels (shape->pixbuf),
-			       gdk_pixbuf_get_width (shape->pixbuf),
-			       gdk_pixbuf_get_height (shape->pixbuf),
-			       gdk_pixbuf_get_rowstride (shape->pixbuf));
+	if (gdk_pixbuf_get_has_alpha (shape->pixbuf)) {
+		gnome_print_rgbaimage (context,
+				       gdk_pixbuf_get_pixels (shape->pixbuf),
+				       gdk_pixbuf_get_width (shape->pixbuf),
+				       gdk_pixbuf_get_height (shape->pixbuf),
+				       gdk_pixbuf_get_rowstride (shape->pixbuf));
+	}
+	else {
+		gnome_print_rgbimage (context,
+				      gdk_pixbuf_get_pixels (shape->pixbuf),
+				      gdk_pixbuf_get_width (shape->pixbuf),
+				      gdk_pixbuf_get_height (shape->pixbuf),
+				      gdk_pixbuf_get_rowstride (shape->pixbuf));
+	}
 }
 
 static void
