@@ -196,7 +196,7 @@ render_path (DiaShape *shape, DiaExportSVGWriter writer, gpointer data)
 		"fill:#%06x;fill-opacity:%f;fill-rule:evenodd",
 		shape->color >> 8,
 		((gdouble) DIA_COLOR_ALPHA (shape->color)) / 255.0,
-		path->line_width,
+		path->line_width / 2.0,
 		svg_linecap[path->cap],
 		svg_linejoin[path->join],
 		path->fill_color >> 8,
@@ -233,7 +233,7 @@ render_ellipse (DiaShape *shape, DiaExportSVGWriter writer, gpointer data)
 		ellipse->height / 2.0,
 		shape->color >> 8,
 		((gdouble) DIA_COLOR_ALPHA (shape->color)) / 255.0,
-		ellipse->line_width,
+		ellipse->line_width / 2.0,
 		ellipse->fill_color >> 8,
 		((gdouble) DIA_COLOR_ALPHA (ellipse->fill_color)) / 255.0);
 }
@@ -317,7 +317,7 @@ render_text (DiaShape *shape, DiaExportSVGWriter writer, gpointer data)
 	writer (data, "<g transform=\"matrix(%f %f %f %f %f %f)\""
 		      " style=\"font-size:%dpx;font-family:%s;font-style:%s;"
 		      "font-stretch:%s;font-weight:%d;font-variant:%s;"
-		      "stroke:#%06x;opacity:%f;text-anchor:%s\">",
+		      "stroke:none;opacity:%f;text-anchor:%s\">",
 		text->affine[0], text->affine[1], text->affine[2],
 		text->affine[3], text->affine[4], text->affine[5],
 		pango_font_description_get_size (font_desc) / PANGO_SCALE,
@@ -326,7 +326,6 @@ render_text (DiaShape *shape, DiaExportSVGWriter writer, gpointer data)
 		svg_stretch[pango_font_description_get_stretch (font_desc)],
 		pango_font_description_get_weight (font_desc),
 		svg_variant[pango_font_description_get_variant (font_desc)],
-		shape->color >> 8,
 		((gdouble) DIA_COLOR_ALPHA (shape->color)) / 255.0,
 		text_anchor);
 
