@@ -37,7 +37,8 @@ enum {
 	PROP_DISCONNECT,
 	PROP_CONNECTED_TO,
 	PROP_CONNECTABLE,
-	PROP_MOVABLE
+	PROP_MOVABLE,
+	PROP_VISIBLE
 };
 
 static gint _dia_handle_size = 9;
@@ -174,7 +175,14 @@ dia_handle_class_init (DiaHandleClass *klass)
 					 PROP_MOVABLE,
 					 g_param_spec_boolean ("movable",
 						_("Movable"),
-						_("Handle can be moved independend from the owner."),
+						_("Handle can be moved independent from the owner."),
+						TRUE,
+						G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_VISIBLE,
+					 g_param_spec_boolean ("visible",
+						_("Visible"),
+						_("Handle is visible by user."),
 						TRUE,
 						G_PARAM_READWRITE));
 	
@@ -187,6 +195,7 @@ dia_handle_init (DiaHandle *handle)
 	handle->need_update_w2i = FALSE;
 	handle->movable = FALSE;
 	handle->connectable = FALSE;
+	handle->visible = TRUE;
 
 	handle->owner = NULL;
 
@@ -332,6 +341,13 @@ dia_handle_set_property (GObject *object, guint property_id,
 						      "movable");
 		handle->movable = g_value_get_boolean (value);
 		break;
+	case PROP_VISIBLE:
+		if (handle->owner && handle->owner->canvas)
+			dia_canvas_preserve_property (handle->owner->canvas,
+						      G_OBJECT (handle),
+						      "visible");
+		handle->visible = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -378,6 +394,9 @@ dia_handle_get_property (GObject *object, guint property_id,
 	case PROP_MOVABLE:
 		g_value_set_boolean (value, handle->movable);
 		break;
+	case PROP_VISIBLE:
+		g_value_set_boolean (value, handle->visible);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 	}
