@@ -63,12 +63,14 @@ G_BEGIN_DECLS
  * 					object the handle is connected to.
  * connectable	boolean		rw	Handle can be connected to another item.
  * movable	boolean		rw	Handle can be moved by a user.
+ * visible	boolean		rw	Handle is visible by a user.
  */
 struct _DiaHandle {
 	GObject object;
 
 	gint movable: 1;
 	gint connectable: 1;
+	gint visible: 1;
 
 	gint need_update_w2i: 1;
 
