@@ -107,7 +107,7 @@ dia_item_tool_button_press (DiaTool *tool, DiaCanvasView *view,
 	DiaCanvasItem *item = NULL;
 	gboolean result = FALSE;
 
-	g_message(G_STRLOC);
+	//g_message(G_STRLOC);
 
 	itool->current_item = NULL;
 
