@@ -567,70 +567,16 @@ ellipse_update (DiaShapeEllipse *shape, DiaCanvasViewItem *item,
 typedef struct {
 	ArtIRect bb;
 	gdouble affine[6];
-	FT_Bitmap bitmap;
-	GSList *selection_box; /* List of ArtSVP's */
+	//FT_Bitmap bitmap;
+	//GSList *selection_box; /* List of ArtSVP's */
+	PangoLayout *layout;
+	gdouble width;
+	gdouble height;
+	gdouble offset_x;
 } TextHelper;
 
 static void text_free (DiaCanvasViewItem *item, DiaShapeViewInfo *view_info);
 
-#if 0
-/* TODO: Remove this function, create a default function instead! */
-static gchar*
-next_char (const gchar *str, gboolean markup)
-{
-        if (markup) {
-                switch (*str) {
-                case '<':
-                        while (*str == '<') {
-                                while (*str != '>')
-                                        str = g_utf8_next_char (str);
-                                /* Point to charachter after the '>' */
-                                str = g_utf8_next_char (str);
-                        }
-                        break;
-                case '&':
-                        /* Move to the end of the &xx; encoded character. */
-                        while (*str != ';')
-                                str = g_utf8_next_char (str);
-                        break;
-                default:
-			break;
-                }
-        }
-        return g_utf8_next_char (str);
-}
-
-static inline void
-add_newlines_to_layout(PangoLayout *layout, const gchar *text,
-		       gboolean markup)
-{
-	GSList *lines;
-	PangoLayoutLine *line;
-	GString *new_text = g_string_new(text);
-	gint index, trailing;
-	const char *pango_text;
-	char *p = new_text->str;
-	gint i;
-
-	pango_text = pango_layout_get_text (layout);
-	for (lines = pango_layout_get_lines (layout);
-	     lines && lines->next; lines = lines->next) {
-		line = lines->data;
-		for (i = line->length; i > 0; i--)
-			p = next_char (p, markup);
-		if (*p != '\n') {
-			g_string_insert_c (new_text, p - new_text->str, '\n');
-			p++;
-		}
-	}
-//	if (markup)
-//		pango_layout_set_markup(layout, new_text->str, new_text->len);
-//	else
-//		pango_layout_set_text(layout, new_text->str, new_text->len);
-	g_string_free (new_text, TRUE);
-}
-#endif
-
 static inline ArtSVP*
 text_update (DiaShapeText *shape, DiaCanvasViewItem *item,
 	     double *affine, ArtSVP *clip, int flags,
@@ -666,67 +612,43 @@ text_update (DiaShapeText *shape, DiaCanvasViewItem *item,
 	if (scale < 1e-4)
 		return NULL;
 
-	/* FixMe: Maybe we can fix the fonts by creating a new PangoContext
-	 * with the right DPI information, instead of using 100dpi. */
-//	if (scale > 1.0) {
-//		max_width = (gint) ceil (shape->max_width * scale);
-//		max_height = (gint) ceil (shape->max_height * scale);
-//		/* compensate for the scaling of the font: */
-//		art_affine_scale(tmp_a, 1 / scale, 1 / scale);
-//		art_affine_multiply (helper->affine, tmp_a, helper->affine);
-//		dia_shape_text_pretty_fill_pango_layout ((DiaShape*) shape,
-//							 layout, scale);
-//	} else {
-//	}
+	layout = helper->layout;
+	if (!layout) {
+		layout = dia_shape_art_get_pango_layout ();
+		helper->layout = layout;
+	}
 
-	layout = dia_shape_art_get_pango_layout ();
 	max_width = (gint) ceil (shape->max_width);
 	max_height = (gint) ceil (shape->max_height);
 	dia_shape_text_fill_pango_layout ((DiaShape*) shape, layout);
 
-	//g_message (G_STRLOC"2ctx=%f scale=%f", 100.0 * scale, scale);
-	//context = pango_ft2_get_context (10.0 * scale, 100.0 * scale);
-	//layout = pango_layout_new (context);
-	//g_object_unref (context);
-
-	//max_width = (gint) ceil (shape->max_width * scale);
-	//max_height = (gint) ceil (shape->max_height * scale);
-	/* compensate for the scaling of the font: */
-	//art_affine_scale(tmp_a, 1 / scale, 1 / scale);
-	//art_affine_multiply (helper->affine, tmp_a, helper->affine);
-	//dia_shape_text_fill_pango_layout ((DiaShape*) shape,
-	//				  layout, 1.0);
-	
-	//add_newlines_to_layout (layout, shape->text, shape->markup);
-
 	/* Create a bitmap to render the text on: */
 	pango_layout_get_pixel_size (layout, &width, &height);
-	//pango_layout_get_size (layout, &width, &height);
-	//width = (gint) ceil ((((gdouble) width) / PANGO_SCALE) * scale);
-	//height = (gint) ceil ((((gdouble) height) / PANGO_SCALE) * scale);
 
 	dwidth = max_width - width;
-	height = MIN (height, max_height);
 	width = MIN (width, max_width);
-	if ((height != helper->bitmap.rows)
-	    || (width != helper->bitmap.width)) {
-		helper->bitmap.rows =  height;
-		helper->bitmap.width = width;
-		/* pitch does the same as rowstride for LibArt functions. */
-		helper->bitmap.pitch = (helper->bitmap.width+3)&~3;
-		if (helper->bitmap.buffer) {
-			helper->bitmap.buffer = g_realloc (helper->bitmap.buffer,
-					helper->bitmap.rows * helper->bitmap.pitch);
-			memset (helper->bitmap.buffer, 0,
-				helper->bitmap.rows * helper->bitmap.pitch);
-		} else {
-			helper->bitmap.buffer = g_malloc0 (helper->bitmap.rows * helper->bitmap.pitch);
-		}
-		helper->bitmap.num_grays = 256;
-		helper->bitmap.pixel_mode = ft_pixel_mode_grays;
-	} else
-		memset (helper->bitmap.buffer, 0,
-			helper->bitmap.rows * helper->bitmap.pitch);
+	height = MIN (height, max_height);
+	helper->width = width;
+	helper->height = height;
+//	if ((height != helper->bitmap.rows)
+//	    || (width != helper->bitmap.width)) {
+//		helper->bitmap.rows =  height;
+//		helper->bitmap.width = width;
+//		/* pitch does the same as rowstride for LibArt functions. */
+//		helper->bitmap.pitch = (helper->bitmap.width+3)&~3;
+//		if (helper->bitmap.buffer) {
+//			helper->bitmap.buffer = g_realloc (helper->bitmap.buffer,
+//					helper->bitmap.rows * helper->bitmap.pitch);
+//			memset (helper->bitmap.buffer, 0,
+//				helper->bitmap.rows * helper->bitmap.pitch);
+//		} else {
+//			helper->bitmap.buffer = g_malloc0 (helper->bitmap.rows * helper->bitmap.pitch);
+//		}
+//		helper->bitmap.num_grays = 256;
+//		helper->bitmap.pixel_mode = ft_pixel_mode_grays;
+//	} else
+//		memset (helper->bitmap.buffer, 0,
+//			helper->bitmap.rows * helper->bitmap.pitch);
 
 	/* Set the width to the buffers width. */
 	if (shape->alignment != PANGO_ALIGN_LEFT) {
@@ -737,41 +659,42 @@ text_update (DiaShapeText *shape, DiaCanvasViewItem *item,
 
 	/* Change offset depending on shape->alignment */
 	if (shape->alignment == PANGO_ALIGN_LEFT) {
-		offset_x = 0;
+		helper->offset_x = 0;
 	} else if (shape->alignment == PANGO_ALIGN_CENTER) {
 		gdouble a[6];
 		art_affine_translate (a, (double) MAX(dwidth, 0) / 2.0, 0.0);
 		art_affine_multiply (helper->affine, a, helper->affine);
-		offset_x = MIN(dwidth, 0) / 2;
+		helper->offset_x = MIN(dwidth, 0) / 2;
 	} else if (shape->alignment == PANGO_ALIGN_RIGHT) {
 		gdouble a[6];
 		art_affine_translate (a, (double) MAX(dwidth, 0), 0.0);
 		art_affine_multiply (helper->affine, a, helper->affine);
-		offset_x = MIN(dwidth, 0);
+		helper->offset_x = MIN(dwidth, 0);
 	} else
 		g_assert_not_reached ();
 	
 	/* Render the text: The result is not nice for scale factors > 1.0,
 	 * since we resized the font. */
-	pango_ft2_render_layout (&helper->bitmap, layout, offset_x, 0);
+//	pango_ft2_render_layout (&helper->bitmap, layout, offset_x, 0);
 
 	/* And we also want text to be transparent, do we adjust the bitmap
 	 * produced by pango_ft2_render_layout().
 	 *
 	 * FixMe: is this a nice fuction to apply alpha to text?
 	 */
-	alpha = shape->shape.color & 0xFF;
-	if (alpha != 0xFF) {
-		for (r = 0; r < helper->bitmap.rows; r++) {
-			for (c = 0; c < helper->bitmap.width; c++) {
-				a = helper->bitmap.buffer[r*helper->bitmap.pitch+c];
-				if (a != 0) {
-					helper->bitmap.buffer[r * helper->bitmap.pitch + c] = a - MIN (255 - alpha, a);
-				}
-			}
-		}
-	}
+//	alpha = shape->shape.color & 0xFF;
+//	if (alpha != 0xFF) {
+//		for (r = 0; r < helper->bitmap.rows; r++) {
+//			for (c = 0; c < helper->bitmap.width; c++) {
+//				a = helper->bitmap.buffer[r*helper->bitmap.pitch+c];
+//				if (a != 0) {
+//					helper->bitmap.buffer[r * helper->bitmap.pitch + c] = a - MIN (255 - alpha, a);
+//				}
+//			}
+//		}
+//	}
 
+	/* Mark the region as dirty. */
 	dr.x0 = 0.0;
 	dr.y0 = 0.0;
 	dr.x1 = (gdouble) width;
@@ -781,13 +704,18 @@ text_update (DiaShapeText *shape, DiaCanvasViewItem *item,
 
 	art_drect_to_irect (&helper->bb, &dr);
 
+#ifdef ENABLE_DEBUG
+	g_message ("Cleaning box (%d, %d, %d, %d)",
+				     helper->bb.x0, helper->bb.y0,
+				     helper->bb.x1, helper->bb.y1);
+#endif
 	gnome_canvas_request_redraw (GNOME_CANVAS_ITEM (item)->canvas,
 				     helper->bb.x0, helper->bb.y0,
 				     helper->bb.x1, helper->bb.y1);
 	view_info->data = helper;
 	view_info->free_func = text_free;
 
-	g_object_unref (layout);
+	//g_object_unref (layout);
 
 	return NULL;
 }
@@ -797,25 +725,63 @@ text_render (DiaShape *shape, DiaCanvasViewItem *item,
 	     GnomeCanvasBuf *buf, DiaShapeViewInfo *view_info)
 {
 	TextHelper *helper = view_info->data;
+	FT_Bitmap bitmap;
+	PangoContext *context;
+	PangoMatrix matrix;
+	gdouble affine[6];
+	gdouble x0 = buf->rect.x0, y0 = buf->rect.y0;
 
 	if (!view_info->data)
 		return;
 
+	bitmap.rows = buf->rect.y1 - buf->rect.y0;
+	bitmap.width = buf->rect.x1 - buf->rect.x0;
+	/* Set pitch boundary on a 4 byte limit. */
+	bitmap.pitch = (bitmap.width + 3) & ~3;
+	bitmap.num_grays = 256;
+	bitmap.pixel_mode = ft_pixel_mode_grays;
+	bitmap.palette_mode = 0;
+	bitmap.palette = NULL;
+	bitmap.buffer = g_malloc0 (bitmap.rows * bitmap.pitch);
+
+	context = pango_layout_get_context (helper->layout);
+
+	matrix.xx = helper->affine[0];
+	matrix.yx = helper->affine[1];
+	matrix.xy = helper->affine[2];
+	matrix.yy = helper->affine[3];
+	matrix.x0 = helper->affine[4] - buf->rect.x0;
+	matrix.y0 = helper->affine[5] - buf->rect.y0;
+	art_affine_identity (affine);
+
+	pango_context_set_matrix (context, &matrix);
+	
+	/*g_message ("Rendering text on (%d, %d)", buf->rect.x0, buf->rect.y0);
+	g_message ("	affine = [%f, %f, %f, %f, %f, %f]",
+			helper->affine[0], helper->affine[1], helper->affine[2],
+			helper->affine[3], helper->affine[4], helper->affine[5]);
+			*/
+	pango_ft2_render_layout (&bitmap, helper->layout, 0, 0);
+
+	// TODO: clip buffer on bounding box boundries
 	art_rgb_a_affine (buf->buf,
-			buf->rect.x0, buf->rect.y0,
-			buf->rect.x1, buf->rect.y1,
+			0, 0,
+			buf->rect.x1 - buf->rect.x0, buf->rect.y1 -buf->rect.y0,
 			buf->buf_rowstride,
-			helper->bitmap.buffer, helper->bitmap.width,
-			helper->bitmap.rows, helper->bitmap.pitch,
+			bitmap.buffer, bitmap.width,
+			bitmap.rows, bitmap.pitch,
 			shape->color >> 8,
-			helper->affine, ART_FILTER_NEAREST, NULL);
+			affine, ART_FILTER_NEAREST, NULL);
+
+	g_free(bitmap.buffer);
 }
 
 static void
 text_free (DiaCanvasViewItem *item, DiaShapeViewInfo *view_info)
 {
 	TextHelper *helper = (TextHelper*) view_info->data;
-	g_free (helper->bitmap.buffer);
+	//g_free (helper->bitmap.buffer);
+	g_object_unref (helper->layout);
 	g_free (view_info->data);
 	view_info->data = NULL;
 }
