@@ -964,9 +964,13 @@ dia_shape_text_set_font_description (DiaShape *shape,
 		pango_font_description_free (text->font_desc);
 		text->font_desc = NULL;
 	}
-	if (font_desc != NULL)
+	if (font_desc != NULL) {
 		text->font_desc = pango_font_description_copy (font_desc);
-
+		if (!pango_font_description_get_size_is_absolute(text->font_desc)) {
+			double size = (double)pango_font_description_get_size(text->font_desc);
+			pango_font_description_set_absolute_size(text->font_desc, size);
+		}
+	}
 	request_update (shape);
 }
 
