@@ -128,6 +128,10 @@ class PlacementTool(diacanvas.view.Tool):
             #print glue_to, sys.getrefcount(glue_to)
 
             if glue_to and (dist <= self.handle_tool.glue_distance):
+                # before connection determine point, which handle should
+                # point to
+                d, (x, y) = glue_to.on_glue(first, wx, wy)
+                first.set_pos_w(float(x), float(y))
                 glue_to.connect_handle(first)
             self.handle_tool.set_grabbed_handle(last)
         elif isinstance(item, diacanvas.CanvasElement):
@@ -138,4 +142,5 @@ class PlacementTool(diacanvas.view.Tool):
         else:
             self.print_error('No handle handling for element %s' % item)
 
-gobject.type_register(PlacementTool)
+# No longer needed for PyGTK 2.8 +
+#gobject.type_register(PlacementTool)
