@@ -64,7 +64,7 @@ static gdouble	pydia_canvas_item_glue			(DiaCanvasItem *item,
 							 gdouble *wx,
 							 gdouble *wy);
 static gboolean pydia_canvas_item_event			(DiaCanvasItem *item,
-							 DiaEvent *event);
+							 gpointer event);
 static void	pydia_canvas_item_move			(DiaCanvasItem *item,
 							 gdouble dx, gdouble dy,
 							 gboolean interactive);
@@ -103,7 +103,7 @@ pydia_canvas_item_update (DiaCanvasItem *item, gdouble affine[6])
 	GValue v = { 0, };
 
 #ifdef ENABLE_DEBUG
-	g_message (__FUNCTION__);
+	g_message (G_STRLOC);
 #endif
 
 	pyg_block_threads();
@@ -155,7 +155,7 @@ pydia_canvas_item_get_shape_iter (DiaCanvasItem *item, DiaCanvasIter *iter)
 	PyObject *py_iter;
 
 #ifdef ENABLE_DEBUG2
-	g_message (__FUNCTION__);
+	g_message (G_STRLOC);
 #endif
 
 	pyg_block_threads();
@@ -190,7 +190,7 @@ static gboolean
 pydia_canvas_item_shape_next (DiaCanvasItem *item, DiaCanvasIter *iter)
 {
 #ifdef ENABLE_DEBUG2
-	g_message (__FUNCTION__);
+	g_message (G_STRLOC);
 #endif
 
 	/* remove reference to previously iterated shape */
@@ -210,7 +210,7 @@ pydia_canvas_item_shape_value (DiaCanvasItem *item, DiaCanvasIter *iter)
 	DiaShape *shape = NULL;
 
 #ifdef ENABLE_DEBUG
-	g_message (__FUNCTION__": data=%p/%p", iter->data[0], iter->data[1]);
+	g_message (G_STRLOC": data=%p/%p", iter->data[0], iter->data[1]);
 #endif
 	py_shape = iter->data[1];
 
@@ -235,7 +235,7 @@ pydia_canvas_item_point (DiaCanvasItem *item, gdouble x, gdouble y)
 	gdouble retval = G_MAXDOUBLE;
 
 #ifdef ENABLE_DEBUG
-	g_message (__FUNCTION__);
+	g_message (G_STRLOC);
 #endif
 		
 	pyg_block_threads();
@@ -266,7 +266,7 @@ pydia_canvas_item_point (DiaCanvasItem *item, gdouble x, gdouble y)
 	pyg_unblock_threads();
 
 #ifdef ENABLE_DEBUG
-	g_message (__FUNCTION__": returning %f", retval);
+	g_message (G_STRLOC": returning %f", retval);
 #endif
 	return retval;
 }
@@ -278,7 +278,7 @@ pydia_canvas_item_move (DiaCanvasItem *item, gdouble dx, gdouble dy, gboolean in
 	PyObject *py_ret;
 
 #ifdef ENABLE_DEBUG
-	g_message (__FUNCTION__);
+	g_message (G_STRLOC);
 #endif
 		
 	pyg_block_threads();
@@ -310,7 +310,7 @@ pydia_canvas_item_handle_motion (DiaCanvasItem *item, DiaHandle *handle,
 	int i;
 
 #ifdef ENABLE_DEBUG
-	g_message (__FUNCTION__);
+	g_message (G_STRLOC);
 #endif
 
 	pyg_block_threads();
@@ -372,7 +372,7 @@ pydia_canvas_item_glue (DiaCanvasItem *item, DiaHandle *handle,
 	gdouble retval = G_MAXDOUBLE;
 
 #ifdef ENABLE_DEBUG
-	g_message (__FUNCTION__);
+	g_message (G_STRLOC);
 #endif
 
 	pyg_block_threads();
@@ -441,7 +441,7 @@ pydia_canvas_item_glue (DiaCanvasItem *item, DiaHandle *handle,
 }
 
 static gboolean
-pydia_canvas_item_event (DiaCanvasItem *item, DiaEvent *event)
+pydia_canvas_item_event (DiaCanvasItem *item, gpointer event)
 {
 	PyObject *self;
 	PyObject *py_event;
@@ -449,7 +449,7 @@ pydia_canvas_item_event (DiaCanvasItem *item, DiaEvent *event)
 	gboolean ret = FALSE;
 
 #ifdef ENABLE_DEBUG
-	g_message (__FUNCTION__);
+	g_message (G_STRLOC);
 #endif
 
 	pyg_block_threads();
@@ -484,7 +484,7 @@ pydia_canvas_item_connect (DiaCanvasItem *item, DiaHandle *handle)
 	self = pygobject_new ((GObject*) item);
 
 #ifdef ENABLE_DEBUG
-	g_message (__FUNCTION__);
+	g_message (G_STRLOC);
 #endif
 		
 	pyg_block_threads();
@@ -516,7 +516,7 @@ pydia_canvas_item_disconnect (DiaCanvasItem *item, DiaHandle *handle)
 	gboolean ret = FALSE;
 
 #ifdef ENABLE_DEBUG
-	g_message (__FUNCTION__);
+	g_message (G_STRLOC);
 #endif
 		
 	pyg_block_threads();
@@ -546,7 +546,6 @@ pydia_canvas_item_disconnect (DiaCanvasItem *item, DiaHandle *handle)
  * In order to get access to those methods, the user should have called
  * diacanvas.register_callbacks() on the class. 
  */
-
 #define GET_NON_PYTHON_PARENT_CLASS(klass, callback) \
 	klass = DIA_CANVAS_ITEM_GET_CLASS (self->obj); \
 	while (klass && klass->callback == pydia_canvas_item_ ## callback) \
@@ -718,12 +717,12 @@ _wrap_dia_canvas_item_on_glue(PyGObject *self, PyObject *args, PyObject *kwargs)
 PyObject *
 _wrap_dia_canvas_item_on_event(PyGObject *self, PyObject *args, PyObject *kwargs)
 {
-	static char *kwlist[] = { "event", NULL };
+	//static char *kwlist[] = { "event", NULL };
 	PyObject *py_event, *py_ret;
 	int ret = FALSE;
-	DiaEvent *event = NULL;
-	DiaCanvasItemClass *klass;
-
+	//gpointer event = NULL;
+	//DiaCanvasItemClass *klass;
+#if 0
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs,
 					 "O:DiaCanvasItem.on_event",
 					 kwlist, &py_event))
@@ -739,8 +738,8 @@ _wrap_dia_canvas_item_on_event(PyGObject *self, PyObject *args, PyObject *kwargs
 	GET_NON_PYTHON_PARENT_CLASS(klass, event);
 
 	if (klass->event)
-		ret = klass->event (DIA_CANVAS_ITEM(self->obj), event);
-
+		ret = klass->event (DIA_CANVAS_ITEM(self->obj), (gpointer) event);
+#endif
 	py_ret = ret ? Py_True : Py_False;
 	Py_INCREF(py_ret);
 	return py_ret;
@@ -836,7 +835,7 @@ pydia_canvas_groupable_add (DiaCanvasGroupable *group, DiaCanvasItem *item)
 	self = pygobject_new ((GObject*) group);
 
 #ifdef ENABLE_DEBUG_GROUP
-	g_message (__FUNCTION__": self = %p, obj = %p", self, group);
+	g_message (G_STRLOC": self = %p, obj = %p", self, group);
 #endif
 
 	if (PyObject_HasAttrString (self, METHOD_PREFIX "add")) {
@@ -870,7 +869,7 @@ pydia_canvas_groupable_remove (DiaCanvasGroupable *group, DiaCanvasItem *item)
 	self = pygobject_new ((GObject*) group);
 
 #ifdef ENABLE_DEBUG_GROUP
-	g_message (__FUNCTION__": self = %p, obj = %p", self, group);
+	g_message (G_STRLOC": self = %p, obj = %p", self, group);
 #endif
 	if (PyObject_HasAttrString (self, METHOD_PREFIX "remove")) {
 		PyObject *py_item, *py_ret;
@@ -904,7 +903,7 @@ pydia_canvas_groupable_get_iter (DiaCanvasGroupable *group, DiaCanvasIter *iter)
 	self = pygobject_new ((GObject*) group);
 
 #ifdef ENABLE_DEBUG_GROUP
-	g_message (__FUNCTION__": self = %p, obj = %p", self, group);
+	g_message (G_STRLOC": self = %p, obj = %p", self, group);
 #endif
 	if (PyObject_HasAttrString (self, METHOD_PREFIX "iter")) {
 		PyObject *py_iter;
@@ -936,7 +935,7 @@ static gboolean
 pydia_canvas_groupable_next (DiaCanvasGroupable *group, DiaCanvasIter *iter)
 {
 #ifdef ENABLE_DEBUG_GROUP
-	g_message (__FUNCTION__": self = %p, obj = %p", self, group);
+	g_message (G_STRLOC": self = %p, obj = %p", self, group);
 #endif
 
 	/* remove reference to previously iterated shape */
@@ -955,7 +954,7 @@ pydia_canvas_groupable_value (DiaCanvasGroupable *group, DiaCanvasIter *iter)
 	PyObject *py_item;
 
 #ifdef ENABLE_DEBUG_GROUP
-	g_message (__FUNCTION__": self = %p, obj = %p", self, group);
+	g_message (G_STRLOC": self = %p, obj = %p", self, group);
 #endif
 
 	py_item = iter->data[1];
@@ -975,7 +974,10 @@ pydia_canvas_groupable_value (DiaCanvasGroupable *group, DiaCanvasIter *iter)
  */
 #undef METHOD_PREFIX
 #define METHOD_PREFIX "on_editable_"
-
+static gboolean pydia_canvas_editable_is_editable	(DiaCanvasEditable *editable);
+static DiaShapeText* pydia_canvas_editable_get_editable_shape
+							(DiaCanvasEditable *editable,
+							 gdouble x, gdouble y);
 static void	pydia_canvas_editable_start_editing	(DiaCanvasEditable *editable,
 							 DiaShapeText *text_shape);
 static void	pydia_canvas_editable_editing_done	(DiaCanvasEditable *editable,
@@ -988,11 +990,87 @@ static void	pydia_canvas_editable_text_changed	(DiaCanvasEditable *editable,
 void
 pydia_canvas_editable_init (DiaCanvasEditableIface *iface)
 {
+	iface->is_editable = pydia_canvas_editable_is_editable;
+	iface->get_editable_shape = pydia_canvas_editable_get_editable_shape;
 	iface->start_editing = pydia_canvas_editable_start_editing;
 	iface->editing_done = pydia_canvas_editable_editing_done;
 	iface->text_changed = pydia_canvas_editable_text_changed;
 }
 
+static gboolean
+pydia_canvas_editable_is_editable (DiaCanvasEditable *editable)
+{
+	PyObject *self;
+	PyObject *py_ret;
+	gboolean ret = TRUE;
+
+#ifdef ENABLE_DEBUG_EDIT
+	g_message (G_STRLOC);
+#endif
+	self = pygobject_new ((GObject*) editable);
+
+	if (PyObject_HasAttrString (self, METHOD_PREFIX "is_editable")) {
+		pyg_block_threads();
+
+		py_ret = PyObject_CallMethod (self,
+					      METHOD_PREFIX "is_editable",
+					      "");
+
+		pyg_unblock_threads();
+
+		if (py_ret) {
+			if (!PyObject_IsTrue (py_ret))
+				ret = FALSE;
+			Py_DECREF (py_ret);
+		} else {
+			PyErr_Print ();
+			PyErr_Clear ();
+		}
+	}
+	Py_DECREF (self);
+
+	return ret;
+}
+
+static DiaShapeText*
+pydia_canvas_editable_get_editable_shape (DiaCanvasEditable *editable,
+					  gdouble x, gdouble y)
+{
+	PyObject *self;
+	DiaShapeText *text_shape = NULL;
+
+	self = pygobject_new ((GObject*) editable);
+
+#ifdef ENABLE_DEBUG_EDIT
+	g_message (G_STRLOC": self = %p, obj = %p", self, text_shape);
+#endif
+
+	if (PyObject_HasAttrString (self, METHOD_PREFIX "get_editable_shape")) {
+		PyObject *py_ret;
+
+		pyg_block_threads ();
+
+		py_ret = PyObject_CallMethod (self,
+					      METHOD_PREFIX "get_editable_shape",
+					      "(dd)", x, y);
+
+		pyg_unblock_threads ();
+
+		if (!py_ret) {
+			PyErr_Print ();
+			PyErr_Clear ();
+		} else {
+	     		if (!pyg_boxed_check (py_ret, DIA_TYPE_SHAPE_TEXT))
+				PyErr_SetString (PyExc_TypeError,
+					 METHOD_PREFIX "get_editable_shape() should only return DiaShapeText's.");
+			else
+				text_shape = pyg_boxed_get (py_ret, DiaShapeText);
+			Py_DECREF (py_ret);
+		}
+	}
+	Py_DECREF (self);
+	return text_shape;
+}
 
 static void
 pydia_canvas_editable_start_editing (DiaCanvasEditable *editable,
@@ -1003,7 +1081,7 @@ pydia_canvas_editable_start_editing (DiaCanvasEditable *editable,
 	self = pygobject_new ((GObject*) editable);
 
 #ifdef ENABLE_DEBUG_EDIT
-	g_message (__FUNCTION__": self = %p, obj = %p", self, text_shape);
+	g_message (G_STRLOC": self = %p, obj = %p", self, text_shape);
 #endif
 
 	if (PyObject_HasAttrString (self, METHOD_PREFIX "start_editing")) {
@@ -1040,7 +1118,7 @@ pydia_canvas_editable_editing_done (DiaCanvasEditable *editable,
 	self = pygobject_new ((GObject*) editable);
 
 #ifdef ENABLE_DEBUG_EDIT
-	g_message (__FUNCTION__": self = %p, obj = %p", self, text_shape);
+	g_message (G_STRLOC": self = %p, obj = %p", self, text_shape);
 #endif
 
 	if (PyObject_HasAttrString (self, METHOD_PREFIX "editing_done")) {
@@ -1077,7 +1155,7 @@ pydia_canvas_editable_text_changed (DiaCanvasEditable *editable,
 	self = pygobject_new ((GObject*) editable);
 
 #ifdef ENABLE_DEBUG_EDIT
-	g_message (__FUNCTION__": self = %p, obj = %p", self, text_shape);
+	g_message (G_STRLOC": self = %p, obj = %p", self, text_shape);
 #endif
 
 	if (PyObject_HasAttrString (self, METHOD_PREFIX "text_changed")) {
