@@ -7,9 +7,7 @@ You might prefer this method when for some reason the Python libraries are
 not recognised by the 'onfigure' script.
 """
 
-MAJOR_VERSION = 0
-MINOR_VERSION = 14
-MICRO_VERSION = 2
+MAJOR_VERSION, MINOR_VERSION, MICRO_VERSION = 0, 14, 4
 
 from distutils.command.build import build
 from distutils.core import setup
@@ -23,7 +21,7 @@ except (ImportError, AssertionError):
     raise SystemExit, 'ERROR: Could not find a recent version of pygtk.'
 
 try:
-    import gnome.canvas
+    import gnomecanvas
 except (ImportError, AssertionError):
     raise SystemExit, 'ERROR: Could not find a GnomeCanvas Python binding'
 
@@ -32,6 +30,7 @@ from dsextras import InstallLib, BuildExt, PkgConfigExtension
 from dsextras import Template, TemplateExtension, pkgc_version_check
 from dsextras import getoutput
 
+
 GLOBAL_MACROS += [('DIACANVAS_MAJOR_VERSION', MAJOR_VERSION)]
 GLOBAL_MACROS += [('DIACANVAS_MINOR_VERSION', MINOR_VERSION)]
 GLOBAL_MACROS += [('DIACANVAS_MICRO_VERSION', MICRO_VERSION)]
